#include "lpc2378.h"
#include "print.h"
#include "os.h"

// currentTaskId is a global variable that holds the ID of the current task.
//
// currentSP is a global variable that holds the value of the
// stack pointer for the current stack.

int currentTaskId;    // The ID of the current task
int currentSP;        // The value of the current task's stack pointer

// Array of TaskBlocks. One for each task.
TaskBlock taskBlocks[NUMBER_OF_TASKS];

//
//  Initialize stack.
//  
//  The stack grows down in memory so we want to start by calculating
//  the top of the stack.
//  
//  Then fill in the initial data starting at the top and working down.
//  
//  The important values are the task_address and the initial CPSR.
//  
//  LR doesn't matter since the task is a infinite loop and should not exit.
//  If it doesn't exit, then it doesn't use the LR since it doesn't return.
//  
int InitializeStack(int* stack, void* task_address)
{
	// Calculate the top of the stack.
	int *stkptr = (int*)(stack + STACKSIZE - 1);

	*(stkptr)   = (int)task_address;   // PC
	*(--stkptr) = (int)0x0e0e0e0e;     // R14 - LR
	*(--stkptr) = (int)0x0c0c0c0c;     // R12
	*(--stkptr) = (int)0x0b0b0b0b;     // R11
	*(--stkptr) = (int)0x0a0a0a0a;     // R10
	*(--stkptr) = (int)0x09090909;     // R9
	*(--stkptr) = (int)0x08080808;     // R8
	*(--stkptr) = (int)0x07070707;     // R7
	*(--stkptr) = (int)0x06060606;     // R6
	*(--stkptr) = (int)0x05050505;     // R5
	*(--stkptr) = (int)0x04040404;     // R4
	*(--stkptr) = (int)0x03030303;     // R3
	*(--stkptr) = (int)0x02020202;     // R2
	*(--stkptr) = (int)0x01010101;     // R1
	*(--stkptr) = (int)0x00000000;     // R0
	*(--stkptr) = ARM_SVC_MODE_ARM;    // CPSR

	return (int)(stkptr);
}

//
//  CreateTask initializes a task's stack and its task block.
//
void CreateTask(int id, int* stack, void* task_address)
{
    taskBlocks[id].stackPointer = InitializeStack(stack, task_address);
    taskBlocks[id].delayCount = 0;
}

//
//  StartOS initializes the globals used by the scheduler and switches
//  to the first task.
//
void StartOS(void)
{
    currentTaskId = 0;
    currentSP = taskBlocks[currentTaskId].stackPointer;

    // Run the current task
    SwitchToCurrent();         // This call never returns to this function.
}

//
// Simple task scheduler that decides which task to run.
// It save the current stack pointer to the task block
// then picks a new task to run. If if cannot find a
// task to run it runs the idleTask.
//
void Scheduler()
{
	//>> Protect the body of this function with a critical section

	printString("Scheduler\n");

	// Save the current stack pointer for the current task.
    taskBlocks[currentTaskId].stackPointer = currentSP;

    // Remember which task we start with
    int lastTaskId = currentTaskId;

    // Loop through the task blocks to find the next task to run
    while(1)
    {
    	// currentTaskId is our index into the array of TaskBlocks
    	currentTaskId++;

    	// Loop around when we get to the end but not counting
    	// the idleTask which the last task in the array.
    	if (currentTaskId >= NUMBER_OF_TASKS - 1)
    	{
    		currentTaskId = 0;
    	}

        // If we loop around back to where we started then it means
        // we have not have not found a runnable task so we'll
        // stop looking and run the idle task.
        if (currentTaskId == lastTaskId)
        {
        	currentTaskId = IDLE_TASK;
        	break;
        }

        // If the task is not delayed then it is runnable so
        // we have found our task and can stop looking.
    	if (taskBlocks[currentTaskId].delayCount != 0)
    	{
    		break;
    	}
    }

    currentSP = taskBlocks[currentTaskId].stackPointer;


}

//
//  IrqDispatcher looks up the handler for the interrupt source from the VIC's vector table
//  and calls that handler. The VIC needs to be configured before calling this function.
//  The function initialize_timer_tick initialize the VIC for timer0 interrupts.
//
void IrqDispatcher(void)
{
	FNCT_PTR pfnct;

	// Read the interrupt vector from the VIC
	pfnct = (FNCT_PTR)READREG32(VICADDRESS);

	// Check for null before call it.
	if (pfnct != (FNCT_PTR)0)
	{
		// Call the handler function for the current interrupt
		(*pfnct)();

		// Acknowledge the VIC interrupt
		WRITEREG32(VICADDRESS, 1);
	}
    else
    {
		// Infinite loop if the interrupt vector was not set.
        while (1);
    }
}

//
// TimerIrqHandler handles interrupts caused by the timer.
// It resets the timer then decrements the counter of any
// delay task. Then calls to schedule to switch tasks.
//
void TickHandler(void)
{
	// Reset timer
	WRITEREG32(T0IR, 0xFF);

	// Decrement the delay count for all tasks as needed.
    for(int i=0; i<NUMBER_OF_TASKS; i++)
    {
    	if(taskBlocks[i].delayCount > 0)
    	{
    		taskBlocks[i].delayCount--;
    	}
    }

	Scheduler();
}

//
//  DelayTask sets a task's delayCount causing the scheduler to skip
//  the task for the number of ticks specified.
//  DelayTask also calls ContextSwitch which calls the scheduler and
//  forces an immediate context switch to another task.
//
void DelayTask(int taskId, int numOfTicks)
{
	//>> Protect with a critical section
    taskBlocks[taskId].delayCount = numOfTicks;
    SwitchContext();
}
