
.set INT_DISABLED,           0xC0    /* Disable both FIQ and IRQ. */
.set MODE_IRQ,               0x12    /* IRQ Mode                  */
.set MODE_SVC,               0x13    /* Supervisor Mode           */
.set ARM_THUMB_MODE,         0x20    /* Set THUMB mode. */
.set ARM_ARM_MODE,           0x00    /* Set ARM mode. */


.extern currentSP
.extern IrqDispatcher
.extern Scheduler

.global SimpleIRQHandler

.global SwitchToCurrent
.global OSIrqHandler
.global SwitchContext

.code 32

/*
 *  Switch to current task without saving previous context.
 *  Used to run the first task the first time.
 *
 *  Pops the initial context setup up by the initialize_stack function.
 *  That means this function will act like a return to the task.
 *
 */
SwitchToCurrent:
                                      /* Switch to the current task's stack. */
    ldr   r0, =currentSP              /* Load r0 with the address for currentSP */
    ldr   sp, [r0]                    /* Load sp with the value stored at the address */
                                      /* sp now points to the task's stack. */

                                      /* Set SPSR to the CPSR for the task. */
    ldmfd sp!, {r0}                   /* Pop the first value from the stack into r0. */
                                      /* That value was the CPSR for the task. */
    msr   spsr_cxsf, r0               /* Load SPSR with that CPSR value in r0. */

                                      /* Run task. */
    ldmfd sp!, {r0-r12, lr, pc}^      /* Pop the rest of the stack setting regs and pc for the task */
                                      /* Acts like a call the task. */


/*
 *  Simple IRQ handler that calls hello_world from the SVC mode.
 *
 *  It saves the context on the IRQ stack.
 *  Switches to the SVC mode and calls hello. Resets SVC LR after call.
 *  Switches back to IRQ mode and pops the context and returns.
 */
SimpleIRQHandler:
    sub    lr, lr, #4                      /* LR offset to return from this exception: -4. */
    stmfd  sp!, {r0-r12, lr}               /* Push working registers. */

    /* Switch to SVC mode */

    msr    cpsr_c, #(INT_DISABLED | MODE_SVC)

    stmfd  sp!, {lr}                       /* Save LR before the call. */
    bl     hello_world                     /* Call hello_world. The branch w/ link overwrites LR. */
    ldmfd  sp!, {lr}                       /* Restore LR after the call. */

    /* Switch back to IRQ mode */

    msr    cpsr_c, #(INT_DISABLED | MODE_IRQ)

    ldmfd  sp!, {r0-r12, pc}^              /* Pop the context and returning to interrupted code. */


/*
 *  OS interrupt handler that calls the IRQ dispatcher.
 *
 */
OSIrqHandler:
   // Add your code from the Context Switch assignment.
   // Replace to the call to Scheduler with a call to IrqDispatcher
   //
   // Note: Change in handling SP and currentSP
   // 
   // In this version save sp to currentSP *before* calling IrqDispatcher
   // IrqDispather does not take a parameter and the Scheduler assumes
   // the current SP has been saved to currentSP.
   //
   // See os.c

/*
 * OS context switcher which is called from C code to force a context switch.
 * It saves the current task's context, calls the scheduler
 * then restores the context of the next task.
 *
 */
SwitchContext:
    // This function runs in  Supervisor mode.

    // Push context on to the current tasks stack.
    // Note LR has the address we want to return to
    // so should be the first thing pushed to the stack.

    // Push the rest of the register and LR.
    // Why do we push LR twice?

    // Save CPSR and push it's value to the stack

    // Save sp to currentSP

    // Call the Scheduler with the task's SP
    // The Scheduler will save it, pick a new task, and set currentSP.

    // Pop task's CPSR and restore it to spsr.
    // spsr will be moved to cpsr when we pop the context with ldmfd

    // Pop task's context. Restores regs and cpsr which reenables interrupts.




